const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const tsImportPluginFactory = require('ts-import-plugin');

const conf = {
    source: path.resolve("./src"),
    target: path.resolve("./dist"),
    dev: process.env.NODE_ENV === 'development'
};

const IS_PRODUCTION = !conf.dev;

const plugins = [
    new HtmlWebpackPlugin({ template: 'index.html' }),
    new CleanWebpackPlugin(conf.target, { allowExternal: true }),
];

module.exports = {
    watch: conf.dev,
    mode: IS_PRODUCTION ? 'production' : 'development',
    entry: path.resolve(conf.source, './App.tsx'),
    output: {
        path: conf.target,
        publicPath: "/",
        filename: IS_PRODUCTION ? '[name].[hash].js' : '[name].js',
    },
    plugins,
    optimization: {
        splitChunks: {
            cacheGroups: {
                commons: {
                    test: /[\\/]node_modules[\\/]/,
                    name: "vendors",
                    chunks: "all"
                }
            }
        }
    },
    devtool: IS_PRODUCTION ? "source-map" : "inline-source-map", // todo: decide on the best option
    resolve: {
        extensions: [".ts", ".tsx", ".js"]
    },
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                exclude: /node_modules/,
                loader: "ts-loader",
                options: {
                    transpileOnly: true,
                    getCustomTransformers: () => ({
                        before: [ tsImportPluginFactory({
                            libraryName: 'antd',
                            libraryDirectory: 'es',
                            style: true
                        }) ]
                    })
                }
            },
            {
                test: /\.(s*)css$/,
                use: ['style-loader', 'css-loader', 'sass-loader']
            },
            {
                test: /\.less$/,
                use: [
                    'style-loader',
                    'css-loader',
                    {
                        loader: 'less-loader',
                        options: {
                            javascriptEnabled: true,
                        }
                    }
                ]
            },
            {
                test: /\.(png|jpg|jpeg|gif)$/,
                exclude: /node_modules/,
                use: ['file-loader']
            },
            {
                test: /.(ttf|otf|eot|svg|woff(2)?)(\?[a-z0-9]+)?$/,
                use: [{
                    loader: 'file-loader',
                    options: {
                        name: '[name].[ext]',
                        outputPath: 'fonts/',
                    }
                }]
            },
        ]
    }
};